#include "service.h"
#include <yaml.h>

#include <stdlib.h>
#include <stdio.h>

#ifdef NDEBUG
#undef NDEBUG
#endif
#include <assert.h>

int
main(int argc, char *argv[])
{
    int number;

    if (argc < 2) {
        printf("Usage: %s file1.yaml ...\n", argv[0]);
        return 0;
    }

    for (number = 1; number < argc; number ++)
    {
        FILE *file;
        yaml_parser_t parser;
        yaml_event_t event;
	char* previous = NULL;
	char* pscalar = NULL;
        int done = 0;
        int count = 0;
        int error = 0;

	int has_map=0;
	int wait_content=1;
	int wait_inputs=0;
	int wait_outputs=0;
	int wait_metadata=0;
	int curr_index=0;
	service* my_service=NULL;
	map* ccontent=NULL;
	map* content=NULL;
	map* icontent=NULL;
	map* iotype=NULL;
	map* ocontent=NULL;
	map* metadata=NULL;

        printf("[%d] Parsing '%s': ", number, argv[number]);
        fflush(stdout);

        file = fopen(argv[number], "rb");
        assert(file);

        assert(yaml_parser_initialize(&parser));

        yaml_parser_set_input_file(&parser, file);

        while (!done)
        {
            if (!yaml_parser_parse(&parser, &event)) {
                error = 1;
                break;
            }

	   switch(event.type){
	   	case YAML_STREAM_START_EVENT:
			printf("YAML_STREAM_START_EVENT\n");
			break;
		case YAML_DOCUMENT_START_EVENT:
			printf("YAML_DOCUMENT_START_EVENT\n");
			break;
		case YAML_DOCUMENT_END_EVENT:
			printf("YAML_DOCUMENT_END_EVENT\n");
			break;
		case YAML_SCALAR_EVENT:
			if(strncasecmp(event.data.scalar.value,"DataInputs",strlen(event.data.scalar.value))==0){
				has_map=1;
				fprintf(stderr,"DataInputs\n");
				if(previous!=NULL){
					free(previous);
					previous=NULL;
				}
			}
			if(previous==NULL)
				has_map=0;
			else{
			  has_map=1;
			  if(wait_content==1){
				if(content==NULL)
					content=createMap(previous,event.data.scalar.value);
				else
					addToMap(content,previous,event.data.scalar.value);
			  }else{
			    if(wait_inputs==1 || wait_outputs==1){
			        if(curr_index==3 && 
				   strncasecmp(event.data.scalar.value,"default",strlen(event.data.scalar.value))!=0 &&
				   strncasecmp(event.data.scalar.value,"supported",strlen(event.data.scalar.value))!=0 ){
				if(icontent==NULL)
					icontent=createMap(previous,event.data.scalar.value);
				else
					addToMap(icontent,previous,event.data.scalar.value);
				}else{
					if(curr_index==4 &&
				   	   strncasecmp(previous,"default",strlen(previous))!=0 &&
					   strncasecmp(previous,"supported",strlen(previous))!=0 ){
						if(iotype==NULL)
							iotype=createMap(previous,event.data.scalar.value);
						else
							addToMap(iotype,previous,event.data.scalar.value);
					}else
					has_map=0;

				}
			    }else
			    	if(wait_outputs==1){
				   if(ocontent==NULL)
					ocontent=createMap(previous,event.data.scalar.value);
				   else
					addToMap(ocontent,previous,event.data.scalar.value);
				}
			  }
			}
			if(previous!=NULL){
				free(previous);
				previous=NULL;
			}
			if(pscalar!=NULL){
				free(pscalar);
				pscalar=NULL;
			}
			printf("YAML_SCALAR_EVENT %d (%s)\n",curr_index,event.data.scalar.value);
			if(has_map==0)
				previous=strdup(event.data.scalar.value);
			pscalar=strdup(event.data.scalar.value);
			break;
		case YAML_SEQUENCE_START_EVENT:
			if(previous!=NULL){
			printf("Sequence kind : %s\n",previous);
				if(strcmp(previous,"dataInputs")==0)
					printf("DataInputs parsing start\n");
				else
					 printf("DataOutputs parsing start\n");
			}
			printf("YAML_SEQUENCE_START_EVENT\n");
			break;
		case YAML_SEQUENCE_END_EVENT:

			break;
		case YAML_MAPPING_START_EVENT:
			if(previous!=NULL){
				printf("Mapping kind : %s\n",previous);
			}
			else
			if(pscalar!=NULL)
				previous=strdup(pscalar);
			if(previous!=NULL && strncasecmp(previous,"MetaData",strlen(previous))==0){
				wait_content=0;
				wait_metadata=1;
			}
			if(previous!=NULL && strncasecmp(previous,"DataInputs",strlen(previous))==0){
				wait_content=0;
				wait_inputs=1;
				wait_outputs=0;
				wait_metadata=0;
			}
			if(previous!=NULL && strncasecmp(previous,"DataOutputs",strlen(previous))==0){
				wait_content=0;
				wait_inputs=0;
				wait_outputs=1;
				wait_metadata=0;
			}
			printf("YAML_MAPPING_START_EVENT index : %d %s\n",curr_index,pscalar);
			curr_index++;
			if(curr_index==3){
				if(icontent!=NULL){
					dumpMap(icontent);
					freeMap(&icontent);
					free(icontent);
					icontent=NULL;
				}
				if(previous!=NULL)
					free(previous);
				previous=NULL;
			}
			if(curr_index==4){
				if(iotype!=NULL){
					dumpMap(iotype);
					freeMap(&iotype);
					free(iotype);
					iotype=NULL;
				}
				if(previous!=NULL)
					free(previous);
				previous=NULL;
			}
			printf("YAML_MAPPING_START_EVENT\n");
			break;
		case YAML_MAPPING_END_EVENT:
			if(curr_index==3){
				if(icontent!=NULL){
					dumpMap(icontent);
					freeMap(&icontent);
					free(icontent);
					icontent=NULL;
				}
				if(previous!=NULL)
					free(previous);
				previous=NULL;
			}
			if(curr_index==4){
				if(iotype!=NULL){
					dumpMap(iotype);
					freeMap(&iotype);
					free(iotype);
					iotype=NULL;
				}
				if(previous!=NULL)
					free(previous);
				previous=NULL;
			}
			has_map=0;
			curr_index--;
			printf("YAML_SEQUENCE_END_EVENT\n");
			printf("YAML_MAPPING_END_EVENT\n");
			break;
	   	case YAML_STREAM_END_EVENT:
			done = (event.type == YAML_STREAM_END_EVENT);
			break;
		default: 
			printf("Didn't understand anything\n");
			break;
	   }
            done = (event.type == YAML_STREAM_END_EVENT);

            yaml_event_delete(&event);

            count ++;
        }

	dumpMap(content);
	freeMap(&content);
	free(content);
	fprintf(stderr,"Inputs \n");
	/*my_service=(service*)malloc(sizeof(service));
	my_service->name="DEMO";
	my_service->content=icontent;
	*/
	dumpMap(icontent);
	freeMap(&icontent);
	fprintf(stderr,"Outputs \n");
	dumpMap(ocontent);
	freeMap(&ocontent);
	if(pscalar!=NULL)
		free(pscalar);
	if(previous!=NULL)
		free(previous);
        yaml_parser_delete(&parser);

        assert(!fclose(file));

        printf("%s (%d events)\n", (error ? "FAILURE" : "SUCCESS"), count);
    }

    return 0;
}

